// src/index.ts
import { Plugin } from "zhin";
var zhText = {
  adapter: "\u53EF\u7528\u9002\u914D\u5668",
  bot: "\u53EF\u7528\u673A\u5668\u4EBA",
  scope: "\u53EF\u54CD\u5E94\u6D88\u606F",
  content: "\u95EE\u9898",
  answer: "\u56DE\u590D",
  redirect: "\u662F\u5426\u4E3A\u91CD\u5B9A\u5411",
  regexp: "\u662F\u5426\u6B63\u5219",
  weight: "\u6743\u91CD",
  authorId: "\u4F5C\u8005id",
  authorName: "\u4F5C\u8005\u540D\u79F0"
};
var fixWeight = (list) => {
  const totalWeight = list.reduce((result, item) => result + item.weight, 0);
  let total = 0;
  return list.map((qa) => {
    const weight = qa.weight / totalWeight;
    return {
      ...qa,
      weightReal: [total, total += weight]
    };
  });
};
var getMatchedQuestion = (message) => {
  const qaList = fixWeight(
    qaPlugin.jsondb.get("qa").filter((qa) => {
      if (!qa.adapter || qa.adapter === "*")
        return true;
      return qa.adapter.includes(message.adapter.name);
    }).filter((qa) => {
      if (!qa.bot || qa.bot === "*")
        return true;
      return qa.bot.includes(message.bot.unique_id);
    }).filter((qa) => {
      return qa.scope.includes(message.message_type);
    }).filter((qa) => {
      if (qa.regexp)
        return new RegExp(qa.content).test(message.raw_message);
      return qa.content === message.raw_message;
    })
  );
  if (!qaList.length)
    return void 0;
  const rand = Math.random();
  return qaList.find((qa) => {
    const [min, max] = qa.weightReal;
    return rand >= min && rand <= max;
  });
};
var getAnswer = (message) => {
  const qa = getMatchedQuestion(message);
  if (!qa || !qa.redirect)
    return qa;
  message.raw_message = qa.answer;
  return getAnswer(message);
};
var qaPlugin = new Plugin("\u95EE\u7B54\u7BA1\u7406");
var qaCommand = qaPlugin.command("\u95EE\u7B54 <question:string> <answer:string>").desc("\u6DFB\u52A0\u95EE\u7B54").option("-a <adapter:string> \u53EF\u7528\u9002\u914D\u5668,\u9ED8\u8BA4*", "*").option("-b <bot:string> \u53EF\u7528\u673A\u5668\u4EBA,\u9ED8\u8BA4*", "*").option("-x <regexp:boolean> \u662F\u5426\u6B63\u5219", false).option("-r <redirect:boolean> \u91CD\u5B9A\u5411", false).option("-s <scope:string> \u4F5C\u7528\u57DF(private,group,direct,guild)\uFF0C\u9ED8\u8BA4\u6240\u6709", "private,group,direct,guild").option("-p <weight:number> \u6743\u91CD", 1).action(async ({ message, adapter, prompt, options }, content, answer) => {
  const existQa = qaPlugin.jsondb.find("qa", (q) => {
    return q.content === content;
  });
  if (existQa) {
    const confirmUpdate = await prompt.confirm("\u8BE5\u95EE\u9898\u5DF2\u5B58\u5728\uFF0C\u662F\u5426\u7EE7\u7EED\u6DFB\u52A0\uFF1F");
    if (!confirmUpdate)
      return "\u5DF2\u53D6\u6D88";
  }
  qaPlugin.jsondb.push("qa", {
    content,
    adapter: options.adapter,
    bot: options.bot,
    scope: options.scope,
    regexp: options.regexp,
    redirect: options.redirect,
    weight: options.weight,
    answer,
    authorId: `${adapter.name}:${message.sender?.user_id}`,
    authorName: message.sender?.user_name || message.sender?.user_id + ""
  });
  return `\u95EE\u7B54\u5DF2\u6DFB\u52A0`;
});
qaPlugin.mounted(() => {
  const qaList = qaPlugin.jsondb.get("qa", []);
  if (!qaList)
    qaPlugin.jsondb.set("qa", []);
});
qaCommand.command("\u95EE\u7B54\u5217\u8868").option("-p <page:number> \u9875\u7801", 1).option("-f <full:boolean> \u5168\u4F5C\u7528\u57DF\u67E5\u8BE2", false).action(({ adapter, bot, prompt, message, options }) => {
  const pageSize = 10;
  const qaList = qaPlugin.jsondb.filter("qa", (qa, index) => {
    return options.full || qa.scope.includes(message.message_type);
  }).filter((_, index) => {
    return index >= (options.page - 1) * pageSize && index < options.page * pageSize;
  });
  if (!qaList.length)
    return `\u6CA1\u6709\u66F4\u591A\u95EE\u7B54\u6570\u636E`;
  return qaList.map((qa, index) => {
    return `${(options.page - 1) * pageSize + index + 1}\uFF1A
\u95EE:${qa.content}
\u7B54:${encodeURIComponent(
      qa.answer
    )}
`;
  }).join("\n") + `\u7B2C${options.page}\u9875\uFF0C\u5171${Math.ceil(qaList.length / pageSize)}\u9875`;
});
qaCommand.command("\u4FEE\u6539\u95EE\u7B54 <no:number> [content:string] [answer:string]").option("-a [adapter:string] \u53EF\u7528\u9002\u914D\u5668").option("-b [bot:string] \u53EF\u7528\u673A\u5668\u4EBA,\u9ED8\u8BA4*").option("-x [regexp:boolean] \u662F\u5426\u6B63\u5219").option("-r [redirect:boolean] \u91CD\u5B9A\u5411").option("-s [scope:string] \u4F5C\u7528\u57DF(private,group,direct,guild)").option("-p [weight:number] \u6743\u91CD").action(({ message, adapter, options }, no, content, answer) => {
  const existQa = qaPlugin.jsondb.find("qa", (_, i) => {
    return i === no - 1;
  });
  if (!existQa)
    return `\u95EE\u7B54${no} \u4E0D\u5B58\u5728`;
  const editor = `${adapter.name}:${message.sender?.user_id}`;
  if (editor !== existQa?.authorId)
    return `\u4EC5\u4F5C\u8005\u672C\u4EBA\uFF1A${existQa?.authorName} \u624D\u80FD\u66F4\u6539\u54E6`;
  qaPlugin.jsondb.splice("qa", no - 1, 1, {
    ...existQa,
    content: content || existQa.content,
    answer: answer || existQa.answer,
    ...options
  });
  return `\u95EE\u7B54${no} \u5DF2\u66F4\u65B0`;
});
qaCommand.command("\u5220\u9664\u95EE\u7B54 <no:number>").option("-y <confirm:boolean> \u662F\u5426\u786E\u8BA4", false).action(async ({ adapter, message, options, prompt }, no) => {
  const qa = qaPlugin.jsondb.get(`qa.${no - 1}`);
  if (!qa)
    return `\u95EE\u7B54\u4E0D\u5B58\u5728`;
  if (qa.authorId !== `${adapter.name}:${message.sender?.user_id}`)
    return `\u975E\u4F5C\u8005\u672C\u4EBA(${qa.authorName})\u4E0D\u53EF\u5220\u9664!`;
  const isConfirm = options.confirm || await prompt.confirm("\u786E\u8BA4\u5220\u9664\u5417\uFF1F");
  if (!isConfirm)
    return "\u5DF2\u53D6\u6D88\u5220\u9664";
  qaPlugin.jsondb.splice("qa", no - 1, 1);
  return `\u5DF2\u5220\u9664\u95EE\u7B54\uFF1A${no}`;
});
qaCommand.command("\u95EE\u7B54\u8BE6\u60C5 <no:number>").action((_, no) => {
  const qa = qaPlugin.jsondb.get(`qa.${no - 1}`);
  if (!qa)
    return `\u95EE\u7B54\u4E0D\u5B58\u5728`;
  return `\u95EE\u7B54${no}\uFF1A
${JSON.stringify(
    Object.fromEntries(
      Object.keys(qa).map((key) => {
        return [zhText[key] || key, qa[key]];
      })
    ),
    null,
    2
  )}`;
});
qaPlugin.middleware(async (adapter, bot, message, next) => {
  const beforeMessage = message.raw_message;
  await next();
  const afterMessage = message.raw_message;
  if (beforeMessage !== afterMessage)
    return;
  const qa = getAnswer(message);
  if (!qa)
    return;
  if (!qa.regexp)
    return message.reply(qa.answer);
  const matchArr = message.raw_message.match(new RegExp(qa.content));
  matchArr.forEach((match, idx) => {
    qa.answer = qa.answer.replace(new RegExp(`\\$${idx}`), match);
  });
  return message.reply(qa.answer);
});
var src_default = qaPlugin;
export {
  src_default as default
};
